package net.shadowfacts.phycon.init

import net.minecraft.block.Block
import net.minecraft.block.BlockState
import net.minecraft.block.entity.BlockEntity
import net.minecraft.block.entity.BlockEntityType
import net.minecraft.util.Identifier
import net.minecraft.util.math.BlockPos
import net.minecraft.util.registry.Registry
import net.shadowfacts.phycon.block.extractor.ExtractorBlock
import net.shadowfacts.phycon.block.extractor.ExtractorBlockEntity
import net.shadowfacts.phycon.block.inserter.InserterBlock
import net.shadowfacts.phycon.block.inserter.InserterBlockEntity
import net.shadowfacts.phycon.block.miner.MinerBlock
import net.shadowfacts.phycon.block.miner.MinerBlockEntity
import net.shadowfacts.phycon.block.netinterface.InterfaceBlock
import net.shadowfacts.phycon.block.netinterface.InterfaceBlockEntity
import net.shadowfacts.phycon.block.netswitch.SwitchBlock
import net.shadowfacts.phycon.block.netswitch.SwitchBlockEntity
import net.shadowfacts.phycon.block.p2p.P2PInterfaceBlock
import net.shadowfacts.phycon.block.p2p.P2PInterfaceBlockEntity
import net.shadowfacts.phycon.block.p2p.P2PReceiverBlock
import net.shadowfacts.phycon.block.p2p.P2PReceiverBlockEntity
import net.shadowfacts.phycon.block.redstone_controller.RedstoneControllerBlock
import net.shadowfacts.phycon.block.redstone_controller.RedstoneControllerBlockEntity
import net.shadowfacts.phycon.block.redstone_emitter.RedstoneEmitterBlock
import net.shadowfacts.phycon.block.redstone_emitter.RedstoneEmitterBlockEntity
import net.shadowfacts.phycon.block.terminal.CraftingTerminalBlock
import net.shadowfacts.phycon.block.terminal.CraftingTerminalBlockEntity
import net.shadowfacts.phycon.block.terminal.TerminalBlock
import net.shadowfacts.phycon.block.terminal.TerminalBlockEntity

/**
 * @author shadowfacts
 */
object PhyBlockEntities {

	val INTERFACE = create(::InterfaceBlockEntity, PhyBlocks.INTERFACE)
	val TERMINAL = create(::TerminalBlockEntity, PhyBlocks.TERMINAL)
	val CRAFTING_TERMINAL = create(::CraftingTerminalBlockEntity, PhyBlocks.CRAFTING_TERMINAL)
	val SWITCH = create(::SwitchBlockEntity, PhyBlocks.SWITCH)
	val EXTRACTOR = create(::ExtractorBlockEntity, PhyBlocks.EXTRACTOR)
	val INSERTER = create(::InserterBlockEntity, PhyBlocks.INSERTER)
	val MINER = create(::MinerBlockEntity, PhyBlocks.MINER)
	val REDSTONE_CONTROLLER = create(::RedstoneControllerBlockEntity, PhyBlocks.REDSTONE_CONTROLLER)
	val REDSTONE_EMITTER = create(::RedstoneEmitterBlockEntity, PhyBlocks.REDSTONE_EMITTER)
	val P2P_INTERFACE = create(::P2PInterfaceBlockEntity, PhyBlocks.P2P_INTERFACE)
	val P2P_RECEIVER = create(::P2PReceiverBlockEntity, PhyBlocks.P2P_RECEIVER)

	private fun <T: BlockEntity> create(builder: (BlockPos, BlockState) -> T, block: Block): BlockEntityType<T> {
		return BlockEntityType.Builder.create(builder, block).build(null)
	}

	fun init() {
		register(InterfaceBlock.ID, INTERFACE)
		register(TerminalBlock.ID, TERMINAL)
		register(CraftingTerminalBlock.ID, CRAFTING_TERMINAL)
		register(SwitchBlock.ID, SWITCH)
		register(ExtractorBlock.ID, EXTRACTOR)
		register(InserterBlock.ID, INSERTER)
		register(MinerBlock.ID, MINER)
		register(RedstoneControllerBlock.ID, REDSTONE_CONTROLLER)
		register(RedstoneEmitterBlock.ID, REDSTONE_EMITTER)
		register(P2PInterfaceBlock.ID, P2P_INTERFACE)
		register(P2PReceiverBlock.ID, P2P_RECEIVER)
	}

	private fun register(id: Identifier, type: BlockEntityType<*>) {
		Registry.register(Registry.BLOCK_ENTITY_TYPE, id, type)
	}

}
